// BSP : STM32 boards groups by STMicroelectronics 2.7.1
// Borad select : STM Boards groups / LoRa boars
// Board part number : LoRa-E5 mini (Board to be selected from Tools submenu 'Board part number)

// Shutdown Current　
//    LoRa-E5 mini           75uA　 3.7V to the BAT+ pin (AP2112 Iq=typ.55uA, MBR130LSFT1 Ir=typ.10uA)
//                           10uA   3.7V to the BAT+ pin (XC6201 Iq=typ.2uA, MBR130LSFT1 Ir=typ.10uA) 
//                            3uA   3.7V to the BAT+ pin (XC6201 Iq=typ.2uA, GS1010FL Ir=typ.2nA)
//    Grove LoRa-E5(no LDO)  0.7uA  3.3V to the VCC pin

// 2024/02/20

#include <RadioLib.h>           // 6.4.2  https://github.com/jgromes/RadioLib
#include <STM32LowPower.h>      // 1.2.5  https://github.com/stm32duino/STM32LowPower
#include <STM32RTC.h>           // 1.4.0  https://github.com/stm32duino/STM32RTC

// RM0461 Reference manual Rev 8  37.1.4 IEEE 64-bit unique device ID register (UID64)
#define TX_UID   uint32_t(*((uint32_t*)0x1FFF7580))    // use uid64 dor Slave device UID

// \Arduino15\packages\STMicroelectronics\hardware\stm32\2.7.1
// \variants\STM32WLxx\WL54JCI_WL55JCI_WLE4J(8-B-C)I_WLE5J(8-B-C)I\variant_LORA_E5_MINI.h : 173
STM32WLx radio = new STM32WLx_Module();
static const uint32_t rfswitch_pins[] =
                         {PA4,  PA5,  RADIOLIB_NC};
static const Module::RfSwitchMode_t rfswitch_table[] = {
  {STM32WLx::MODE_IDLE,  {LOW,  LOW}},
  {STM32WLx::MODE_RX,    {HIGH, LOW}},
  {STM32WLx::MODE_TX_HP, {LOW, HIGH}},  // for LoRa-E5 mini, Grove LoRa-E5
//  {STM32WLx::MODE_TX_LP, {HIGH, HIGH}},   // for LoRa-E5-LE mini
  END_OF_MODE_TABLE,
};

#define dataNum   12                // data bytes
union unionData {                   // for data type conversion
  uint32_t  dataBuff32[dataNum/4];
  float     dataBuff32F[dataNum/4];
  uint16_t  dataBuff16[dataNum/2];
  uint8_t   dataBuff8[dataNum];
};
union unionData ud;

// *******************************************************************************************************
void setup() {
  // Set all ports to which components are connected to INPUT
/*  
  pinMode(PB5, INPUT);      // LED RED  LoRa-E5 mini
  pinMode(PA9, INPUT);      // LED RED  Grove LoRa-E5
  pinMode(PB6, INPUT);      // TX
  pinMode(PB7, INPUT);      // RX
  pinMode(PB13, INPUT);     // BOOT SW
  pinMode(PA15, INPUT);     // SDA 4k7 pullup
  pinMode(PB15, INPUT);     // SCL 4k7 pullup
  pinMode(PB4, INPUT);      // A4 battery voltage
  pinMode(PA0, INPUT);      // D0 battery voltage read enable
  pinMode(PA13, INPUT);     // SWDIO
  pinMode(PA14, INPUT);     // SWCLK
*/
  // Radio initialization
  radio.setRfSwitchTable(rfswitch_pins, rfswitch_table);

  // int16_t begin(float freq = 434.0, float bw = 125.0, uint8_t sf = 9, uint8_t cr = 7, \
  // uint8_t syncWord = RADIOLIB_SX126X_SYNC_WORD_PRIVATE, int8_t power = 10, \
  // uint16_t preambleLength = 8, float tcxoVoltage = 1.6, bool useRegulatorLDO = false);
  int state = radio.begin(922.6, 125.0, 12, 5, 0x12, 13, 8, 1.7, 0);

  // Configure low power
  LowPower.begin();
}

// ***********************************************************************************************************
void loop() 
{
  // Example transmitting data
  ud.dataBuff32[0] = TX_UID;                      // TX Device UID
  ud.dataBuff32[1] = millis();
  ud.dataBuff32[2] = micros();

  int state = radio.transmit(ud.dataBuff8, dataNum);

  // Shutdown
  LowPower.shutdown(100000);
}
